unit frmFile;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, StdCtrls, MMSystem, DDirectSound, DSound, Wave;

type
  TFileForm = class(TForm)
    labelName: TLabel;
    labelStatus: TLabel;  
    Bevel1: TBevel;
    Bevel2: TBevel;
    labelPlay: TLabel;
    Bevel3: TBevel;
    labelWrite: TLabel;
    Bevel4: TBevel;
    labelFreq: TLabel;
    trackbarFreq: TTrackBar;
    labelPan: TLabel;
    trackbarPan: TTrackBar;
    Bevel5: TBevel;
    Bevel6: TBevel;
    labelVol: TLabel;
    trackbarVol: TTrackBar;
    trackbarVolL: TTrackBar;
    trackbarVolR: TTrackBar;
    Label2: TLabel;
    Label3: TLabel;
    Bevel7: TBevel;
    buttonPlay: TButton;
    buttonClose: TButton;
    checkboxLooped: TCheckBox;
    Bevel8: TBevel;
    procedure FormDestroy(Sender: TObject);
    procedure trackbarVolChange(Sender: TObject);
    procedure trackbarFreqChange(Sender: TObject);
    procedure trackbarPanChange(Sender: TObject);
    procedure checkboxLoopedClick(Sender: TObject);
    procedure buttonPlayClick(Sender: TObject);
    procedure buttonCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
  public
    pbData: PBYTE;               // Pointer to actual data of file.
    cbSize: Longint;             // Size of data.
    pwfx: PWAVEFORMATEX;          // Pointer to waveformatex structure.

    dwFreq: Longint;  // Frequency.
    dwPan: Longint;          // Panning info.
    dwVol: Longint;          // Total volume.
    fLooped: Boolean;        // Looped?

    fPlaying: Boolean;       // Is this one playing?
    fLost: Boolean;          // Is this one lost?
    fHardware: Boolean;      // Is this a hardware buffer?
    fSticky: Boolean;        // Is this a sticky buffer?

    DSB: TDirectSoundBuffer; // Pointer to direct sound buffer.

    nFileName: Integer;      // Index to filename, not including dir.
    FileName: String;

    procedure CreateControl;
    procedure StartDSound;
    procedure StopDSound;
    procedure NewDirectSoundBuffer;
    procedure ReleaseDirectSoundBuffer;
    procedure UpdateStatus( dwStatus: Longint );
    procedure UpdateLRVolume;
    procedure ChangeOutputFreq;
    procedure ChangeOutputPan;
    procedure ChangeOutputVol;
    procedure SetAllText;
    procedure Init;
  end;


implementation

{$R *.DFM}

uses frmMain;



// =======================================================================
//  This function updates the status window by writing the specified
//  string to the window, prepended by a string indicating whether
//  the buffer is in hardware or software
// =======================================================================
procedure TFileForm.UpdateStatus( dwStatus: Longint );
var
  st: String;
begin
  if fHardware then st := 'HW-'
  else st := 'SW-';

  if (dwStatus and DSBSTATUS_BUFFERLOST <> 0) then
    st := st + 'Lost'
  else if (dwStatus and DSBSTATUS_PLAYING <> 0) then
    st := st + 'Playing'
    else
      st := st + 'Stopped';


  if (fSticky) then st := st + 'Sticky';
  labelStatus.Caption := st;

  labelPlay.Caption := 'Play ' + IntToStr(DSB.PlayPosition);
  labelWrite.Caption := 'Write ' + IntToStr(DSB.WritePosition);
end;



procedure TFileForm.CreateControl;
var
  MinFreq, MaxFreq: Longint;
begin
  labelName.Caption := Copy(FileName, nFileName+1, 200);
	// get the min and max range that the sound card supports.
	// If the buffer is in hardware query the card, else use
	// our ifdef'd values.
	if (fHardware) then
	begin
		MinFreq := MainForm.DS.Caps.dwMinSecondarySampleRate;
		MaxFreq := MainForm.DS.Caps.dwMaxSecondarySampleRate;
  end
	else
	begin
		MinFreq := DSBFREQUENCY_MIN;
		MaxFreq := DSBFREQUENCY_MAX;
	end;

  trackbarFreq.Min := MinFreq div FREQFACTOR;
  trackbarFreq.Max := MaxFreq div FREQFACTOR;
  trackbarFreq.Position := (dwFreq + FREQADD) div FREQFACTOR;

  trackbarPan.Position := dwPan;

  trackbarVol.Position := dwVol;
  dwVol := MAXVOL_TB - dwVol;

  SetAllText;
  UpdateLRVolume;
end;


// =======================================================================
//  This routine will initialize a new direct sound buffer,
//    set the data in the buffer,
//    set the rate, format, etc...
// =======================================================================
procedure TFileForm.NewDirectSoundBuffer;
var
  bd, bd2: Pointer;
  dsbd: TDSBufferDesc;
  Len, Len2: DWord;
begin
  try
    // Set up the direct sound buffer.
    ZeroMemory( @dsbd, sizeof(dsbd) );
    dsbd.dwSize := sizeof(TDSBUFFERDESC);
    dsbd.dwFlags := 0 or DSBCAPS_STATIC;
    // Use new GetCurrentPosition() accuracy (DirectX 2 feature)
    dsbd.dwFlags := dsbd.dwFlags or DSBCAPS_CTRLDEFAULT
      or DSBCAPS_GETCURRENTPOSITION2;

    if (fSticky) then
        dsbd.dwFlags := dsbd.dwFlags or DSBCAPS_STICKYFOCUS;
    dsbd.dwBufferBytes := cbSize;
    dsbd.lpwfxFormat := pwfx;

    DSB := TDirectSoundBuffer.Create( MainForm.DS, dsbd );

    // Ok, lock the sucker down, and copy the memory to it.
    DSB.Lock( 0, cbSize, bd, Len, bd2, Len2, 0 );
    try
      CopyMemory( bd, pbData, cbSize );
    finally
      // Ok, now unlock the buffer, we don't need it anymore.
      DSB.Unlock( bd, cbSize, nil, 0 );
    end;

    bd := nil;

    DSB.Volume := MAXVOL_VAL;
    DSB.Pan := MIDPAN_VAL;

    if (DSB.Caps.dwFlags and DSBCAPS_LOCHARDWARE) <> 0 then fHardware := True
    else fHardware := False;
  except
    on EDirectSoundError do
    begin
      DSB.Free;
      DSB := nil;
      raise;
    end;
  end;
end;


// =======================================================================
//  This routine will release a direct sound buffer,
//    freeing up memory, resources,
//    whatever.
// =======================================================================
procedure TFileForm.ReleaseDirectSoundBuffer;
begin
  DSB.Free;
  DSB := nil;
end;


// =======================================================================
//  This routine will set the freq, vol and pan slider text
//    according to the value
//    passed in.
// =======================================================================
procedure TFileForm.SetAllText;
begin
  labelFreq.Caption := 'Freq: ' + IntToStr(dwFreq) + ' Hz';
  labelPan.Caption := 'Pan: ' + IntToStr( (dwPan + SHIFTPAN_TB) * MULTPAN_TB );
  labelVol.Caption := 'Volume: ' + IntToStr( (dwVol + SHIFTVOL_TB) * MULTVOL_TB );
end;


// =======================================================================
//  This routine will update the left and right
//    volume according to main volume
//    and pan.
// =======================================================================
procedure TFileForm.UpdateLRVolume;
var
  volLeft, volRight: Integer;
begin
  if (dwPan < MIDPAN_TB) then
  begin
    volLeft := dwVol;
    volRight := (dwPan*dwVol) div MIDPAN_TB;
  end
  else
  begin
    volLeft := ((dwPan - MAXPAN_TB)* -dwVol) div MIDPAN_TB;
    volRight := dwVol;
  end;

  trackbarVolL.Position := MAXVOL_TB - volLeft;
  trackbarVolR.Position := MAXVOL_TB - volRight;
end;


// =======================================================================
//  This will change the output panning position for a certain FILEINFO.
//    This is
//    done by sending messages to the direct sound driver
// =======================================================================
procedure TFileForm.ChangeOutputPan;
begin
  DSB.Pan :=  (dwPan + SHIFTPAN_TB) * MULTPAN_TB;
end;


// =======================================================================
//  This will change the output freq for a certain FILEINFO.  This is
//    done by sending messages to the direct sound driver
// =======================================================================
procedure TFileForm.ChangeOutputFreq;
begin
  DSB.Frequency := dwFreq;
end;


// =======================================================================
//  This will change the output vol for a certain FILEINFO.  This is
//    done by sending messages to the direct sound driver
// =======================================================================
procedure TFileForm.ChangeOutputVol;
begin
  DSB.Volume := (dwVol + SHIFTVOL_TB) * MULTVOL_TB;
end;


// =======================================================================
//  This routine will start a sound to be played.
//    Input:
// =======================================================================
procedure TFileForm.StartDSound;
begin
  // Already playing?
  if (DSB.Status and DSBSTATUS_PLAYING <> 0) then
  // Don't bother playing, just restart
    DSB.PlayPosition := 0
  else
  begin
    if fLooped then DSB.Play( DSBPLAY_LOOPING )
    else DSB.Play( 0 );
    fPlaying := TRUE;
  end;
end;


// =======================================================================
procedure TFileForm.StopDSound;
begin
    if not fPlaying then exit;

    // Stop sound here...
    DSB.Stop;

    fPlaying := FALSE;
end;


procedure TFileForm.Init;
begin
  Show;

  try
    WaveLoadFile( FileName, cbSize, pwfx, Pointer(pbData) );
  except
    on exception do Close;
  end;

  NewDirectSoundBuffer;

  MainForm.Files.Add( self );

  dwFreq := pwfx^.nSamplesPerSec;
  dwPan := (MAXPAN_TB-MINPAN_TB) div 2;
  dwVol := MINVOL_TB;
  CreateControl;
  ChangeOutputVol;
  ChangeOutputFreq;
  ChangeOutputPan;

  MainForm.UpdateMainStatus;
end;


procedure TFileForm.FormDestroy(Sender: TObject);
begin
  MainForm.Files.Remove(self);
  MainForm.UpdateMainStatus;

  ReleaseDirectSoundBuffer;
  FreeMem( pwfx );
  FreeMem( pbData );
end;

procedure TFileForm.trackbarVolChange(Sender: TObject);
begin
	dwVol := MAXVOL_TB - trackbarVol.Position;
	ChangeOutputVol;
	SetAllText;
	UpdateLRVolume;
end;

procedure TFileForm.trackbarFreqChange(Sender: TObject);
begin
	dwFreq := trackbarFreq.Position * FREQFACTOR - FREQADD;
	ChangeOutputFreq;
	SetAllText;
end;

procedure TFileForm.trackbarPanChange(Sender: TObject);
begin
	dwPan := trackbarPan.Position;
	ChangeOutputPan;
	SetAllText;
  UpdateLRVolume;
end;

procedure TFileForm.checkboxLoopedClick(Sender: TObject);
begin
	fLooped := checkboxLooped.Checked;

	// If it is playing then reset the looping to be proper
	if fPlaying then
    if fLooped then DSB.Play( DSBPLAY_LOOPING )
    else DSB.Play( 0 );
end;

procedure TFileForm.buttonPlayClick(Sender: TObject);
begin
	if fPlaying then
  begin
    StopDSound;
    buttonPlay.Caption := 'Play';
    UpdateStatus( 0 );
  end
  else
  begin
	  StartDSound;
    buttonPlay.Caption := 'Stop';
    UpdateStatus( DSBSTATUS_PLAYING );
  end;
end;

procedure TFileForm.buttonCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TFileForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

end.
