unit frmMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, DDirectSound, DSound, Registry, MMSystem, frmDriver, frmFile,
  CommDlg, frmLatency, frmOutput, Wave;

const
  AppName = 'Direct Sound Demo';
  sMessage = 'Direct Sound Demo';

  MAX_PATH = 255;
// Registry Key and Value names that allow us to retrive a path something like
// "C:\DXSDK\SDK\MEDIA", but matching the current install.
  RegKeyDirect3D = 'Software\Microsoft\Direct3D';
  RegValueD3DPath = 'D3D Path';
  RegKeyDSShow = 'Software\CBarbosa\DSShow';
  RegValueEnumDrivers = 'EnumDrivers';

  IDC_FONEST_STICKY = 111;
  IDD_FILEOPEN_NEST = 110;

  FREQFACTOR  = 5;
  FREQADD     = 0;
  FREQPAGE    = 100;
  PANPAGE     = 5;
  MINFREQ_TB  = 0;
  MAXFREQ_TB  = 100000;
  MINPAN_TB   = 0;
  MIDPAN_TB   = 400;
  MAXPAN_TB   = 800;
  SHIFTPAN_TB = -400;
  MULTPAN_TB  = 10;
  MINPAN_VAL  = -400;
  MIDPAN_VAL  = 0;
  MAXPAN_VAL  = 400;
  MINVOL_TB   = 0;
  MAXVOL_TB   = 400;
  SHIFTVOL_TB = -400;
  MULTVOL_TB  = 10;
  MINVOL_VAL  = -400;
  MAXVOL_VAL  = 0;


type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    miFile: TMenuItem;
    miFileOpen: TMenuItem;
    miFileExit: TMenuItem;
    miOptions: TMenuItem;
    miOptionsOutputType: TMenuItem;
    miOptionsCheckLatency: TMenuItem;
    miOptionsEnumerateDrivers: TMenuItem;
    miHelp: TMenuItem;
    miAbout: TMenuItem;
    Timer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure miAboutClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure miFileExitClick(Sender: TObject);
    procedure miOptionsEnumerateDriversClick(Sender: TObject);
    procedure miOptionsCheckLatencyClick(Sender: TObject);
    procedure miOptionsOutputTypeClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure miFileOpenClick(Sender: TObject);
  private
    Registry: TRegistry;
    CDStartPath: String;	        // The path to start the Open dialog in

    procedure CreateParams(var Params: TCreateParams); override;
  public
    DS: TDirectSound;
    Files: TList;

    procedure ParseCommandLine;
    procedure BatchOpenFiles( FilesFound: TStrings; fPlay, fLoop: Boolean );
    procedure StopAllDSounds;
    procedure UpdateMainStatus;
    function OpenFileDialog( FileName: String; var nFileName: Integer;
      var fSticky: Boolean): Boolean;
    procedure FreeAllList;
    function IsValidWave( FileName: String ): Boolean;
    procedure GetMediaStartPath;
  end;

var
  MainForm: TMainForm;
  pofn: POPENFILENAME;

implementation

uses frmAbout;

{$R *.DFM}


// =======================================================================
// This "hook procedure" is called by the common dialog code for certain
//   events that may occur during the life of our nested dialog structure.
//	 We nest the Explorer style dialog inside our file open dialog so we
//   can addd a check box for stick buffers.
//
// =======================================================================
function FileOpenCustomTemplateDlgProc( hDlg: HWND; Msg: UINT;
  wParam: WPARAM; lParam: LPARAM ): UINT; stdcall;
begin
  case Msg of
    WM_INITDIALOG:
      begin
        pofn := POpenFilename(lParam);
        // Set the flag to match the current state of the check box control */
        PLongint(pofn^.lCustData)^ := SendDlgItemMessage( hDlg, IDC_FONEST_STICKY,
          BM_GETCHECK, 0, 0 );
        Result := 1;
        exit;
      end;

    WM_NOTIFY:
      begin
        case (POfNotify(lParam)^.hdr.code) of
          CDN_FILEOK:
            //* We can do lots of things in this notification message.  The most
            //* important is that we can decide whether the Common Dialog call will
            //* go through or whether it will fail.  I decided to handle the checkbox
            //* control in this one place versus 4 others... -PRN
            //
            PLongint(pofn^.lCustData)^ := SendDlgItemMessage( hDlg, IDC_FONEST_STICKY,
              BM_GETCHECK, 0, 0 );
        end;
      end;
  end;

  Result := 0;
end;




// =======================================================================
// ParseCommandLine()
//
//    Given a command-line string without the module name, this function will
// parse the command line and takes action on whatever it finds there.
// =======================================================================
procedure TMainForm.ParseCommandLine;
var
  Cnt: Integer;
  FilesFound: TStrings;
  fStartPlaying, fStartLooping: Boolean;
begin
  if ParamCount = 0 then exit;

  fStartPlaying := False;
  fStartLooping := False;
  FilesFound := TStringList.Create;

  // First get all the command line switches
  for Cnt := 1 to ParamCount do
    if (ParamStr(Cnt)[1] = '/') or (ParamStr(Cnt)[1] = '-') then
    begin
      if StrLIComp( PChar(Copy(ParamStr(Cnt), 2, 4)), 'PLAY', 4 ) = 0 then
        fStartPlaying := TRUE
      else
        if StrLIComp( PChar(Copy(ParamStr(Cnt), 2, 4)), 'LOOP', 4 ) = 0 then
          fStartLooping := TRUE;
    end
    else break;

  // Anything left on the command-line will be treated as a filename and
  // we'll attempt to open it after we've found them all
  while Cnt <= ParamCount do
  begin
    FilesFound.Add( ParamStr(Cnt) );
    Inc( Cnt );
  end;

  // This function will take the array of strings we've created and open
  // each string as a file.  It will obey the global fStartPlaying and
  // fStartLooping flags we may have already set above
  if (FilesFound.Count > 0) then
    BatchOpenFiles( FilesFound, fStartPlaying, fStartLooping );

  FilesFound.Free;
end;


// =======================================================================
// GetMediaStartPath()                                                      */
//                                                                          */
//   This helper function attempts to get the media directory for Direct3D, */
// which is where all the installed DX wave files go. If it can't find that */
// it settles for the media sub-directory of the Windows directory.         */
// =======================================================================
procedure TMainForm.GetMediaStartPath;
begin
  Registry.RootKey := HKEY_LOCAL_MACHINE;
  if Registry.OpenKey( RegKeyDirect3D, False ) then
    if Registry.ValueExists( RegValueD3DPath ) then
      CDStartPath := Registry.ReadString( RegValueD3DPath )
    else
    begin
      SetLength( CDStartPath, MAX_PATH );
      GetWindowsDirectory( PChar(CDStartPath), MAX_PATH );
      CDStartPath := Copy(CDStartPath, 1, Pos(#0, CDStartPath)-1);
      if (CDStartPath[ Length(CDStartPath) ] <> '\') then
        CDStartPath := CDStartPath + '\';
      CDStartPath := CDStartPath + 'MEDIA';
    end;
  Registry.CloseKey;

  Registry.RootKey := HKEY_CURRENT_USER;
end;



// =======================================================================
// BatchOpenFiles()
//
//    Takes an array of string pointers and tries to open each as a file to
// playback.  If fPlay is TRUE, the files will be played as they are being
// opened.  If fLoop is TRUE, they will also be set to loop.
//
// Returns: FALSE in the event of catastrophic failure, otherwise TRUE.
// =======================================================================
procedure TMainForm.BatchOpenFiles( FilesFound: TStrings; fPlay, fLoop: Boolean );
var
  Cnt: Integer;
  ff: TFileForm;
begin
  for Cnt := 0 to FilesFound.Count-1 do
  begin
    ff := TFileForm.Create(self);
    ff.Parent := self;
    ff.Align := alLeft;
    ff.FileName := FilesFound.Strings[Cnt];
    ff.nFileName := 0;

    ff.Init;

  	// LOOP is only obeyed if PLAY was also specified
  	if fPlay then
    begin
	    if fLoop then
      begin
        ff.fLooped := True;
        ff.checkboxLooped.Checked := True;
      end;
      ff.buttonPlay.OnClick( self );
    end;
  end;

  UpdateMainStatus;
end;



// =======================================================================
//  This routine will stop all the sounds which are playing.
// =======================================================================
procedure TMainForm.StopAllDSounds;
var
  Cnt: Integer;
begin
  for Cnt := 0 to Files.Count-1 do
    TFileForm( Files[Cnt] ).StopDSound;
end;


// =======================================================================
//  This function updates the main window title to show some
//  relevant information about the direct sound object
//
// =======================================================================
procedure TMainForm.UpdateMainStatus;
var
  dsc: TDSCaps;
begin
  // Update main window title with some relevant info
  dsc := DS.Caps;
  Caption := sMessage
  + ' : free hw memory = '
  + IntToStr((dsc.dwFreeHwMemBytes+512) div 1024)
  + 'Kb, free hw buffers = '
  + IntToStr( dsc.dwFreeHwMixingAllBuffers );
end;


// =======================================================================
//  This will pop up the open file dialog and allow the user to pick one file.
//
//    Input:
//	hWnd            -   Handle of parent window.
//	pszFileName         -   String to store filename in, must be at least MAX_PATH long.
//
//
//    Output:
//	TRUE if a file was  picked successfully, else FALSE (user didn't pick a file)
//
//
// =======================================================================
function TMainForm.OpenFileDialog( FileName: String; var nFileName: Integer;
  var fSticky: Boolean): Boolean;
var
  ofn: TOpenFilename;
  fReturn, fValid: Boolean;
begin
  ofn.lStructSize         := sizeof(ofn);
  ofn.hwndOwner           := Handle;
  ofn.hInstance           := HInstance;
  ofn.lpstrFilter         := 'Wave Files' + #0 + '*.wav' + #0 + 'All Files' + #0
    + '*.*' + #0#0;
  ofn.lpstrCustomFilter   := nil;
  ofn.nMaxCustFilter      := 0;
  ofn.nFilterIndex        := 1;
  ofn.lpstrFile           := Pointer(FileName);
  ofn.nMaxFile            := MAX_PATH;
  ofn.lpstrFileTitle      := nil;
  ofn.nMaxFileTitle       := 0;
  ofn.lpstrInitialDir     := Pointer(CDStartPath);
  ofn.lpstrTitle          := 'File Open';
  ofn.Flags               := OFN_FILEMUSTEXIST or OFN_EXPLORER
    or OFN_ENABLETEMPLATE or OFN_ENABLEHOOK or OFN_HIDEREADONLY;
  ofn.nFileOffset         := 0;
  ofn.nFileExtension      := 0;
  ofn.lpstrDefExt         := 'wav';
  ofn.lCustData           := Longint(@fSticky);
  ofn.lpfnHook            := FileOpenCustomTemplateDlgProc;
  ofn.lpTemplateName      := MAKEINTRESOURCE(IDD_FILEOPEN_NEST);

  repeat
    fReturn := GetOpenFileName( ofn );
    if fReturn then
    begin
      // Set the start path for the next time this dialog is opened
      nFileName := ofn.nFileOffset;
      CDStartPath := Copy(FileName, 1, nFileName);

      fValid := IsValidWave( FileName );
      if (not fValid) then
        MessageBox(Handle, 'Wave files must be PCM format!',
          'Invalid Wave File', MB_OK or MB_ICONSTOP)
    end
    else
      fValid := TRUE;         // Force break out of loop.
  until fValid = True;

  Result := fReturn;
end;



// =======================================================================
//  This routine will free the whole linked list in pFileInfoFirst,
//    including all the
//    memory used by the wave file, waveformatex structure, etc.
// =======================================================================
procedure TMainForm.FreeAllList;
begin
  while Files.Count > 0 do
    TFileForm(Files[0]).Free;
end;



// =======================================================================
//  This function will determine if the filename passed
//    in is a valid wave for this
//    app, that is a PCM wave.
//
//    Input:
//	FileName -   FileName to check.
//
//    Output:
//	FALSE if not a valid wave, TRUE if it is.
//
// =======================================================================
function TMainForm.IsValidWave( FileName: String ): Boolean;
var
  wfx: TWaveFormatEx;
  Size, Offset: Longint;
  fs: TFileStream;
begin
  Result := True;
  try
    fs := TFileStream.Create( FileName, fmOpenRead );
    try
      GetStreamWaveInfo( fs, Size, Offset, wfx );

      if (wfx.wFormatTag <> WAVE_FORMAT_PCM) then abort;
    finally
      fs.Free;
    end;
  except
    on Exception do Result := False;
  end;
end;


procedure TMainForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or WS_POPUP;
end;


procedure TMainForm.FormCreate(Sender: TObject);
var
  G: PGUID;
begin
  Registry := TRegistry.Create;

  Files := TList.Create;

  // Set up the global File...Open dialog's start directory
  GetMediaStartPath;

  // Now set up all the direct sound stuff...
  G := nil;
  Registry.OpenKey( RegKeyDSShow, True );
  if Registry.ValueExists( RegValueEnumDrivers ) then
    miOptionsEnumerateDrivers.Checked := Registry.ReadBool( RegValueEnumDrivers );
  Registry.CloseKey;

  G := nil;
  if miOptionsEnumerateDrivers.Checked then
    with TDriverForm.Create( self ) do
      try
        ShowModal;
        if GUIDPtr <> nil then
        begin
          new( G );
          G^ := GUIDPtr^;
        end;
      finally
        Free;
      end;

  DS := TDirectSound.Create( G, 0 );

  if G <> nil then
    Dispose( G );

  // Note we need to set the level to be priority to set the
  // format of the primary buffer
  DS.SetCooperativeLevel( Application.Handle, DSSCL_PRIORITY );

  DS.PrimaryBuffer.Play( DSBPLAY_LOOPING );

  UpdateMainStatus();

  ParseCommandLine;
end;



procedure TMainForm.FormDestroy(Sender: TObject);
begin
  StopAllDSounds;
  FreeAllList;

  Files.Free;

  // Destroy the direct sound
  DS.Free;

  Registry.OpenKey( RegKeyDSShow, True );
  Registry.WriteBool( RegValueEnumDrivers, miOptionsEnumerateDrivers.Checked );
  Registry.CloseKey;
  Registry.Free;
end;


procedure TMainForm.miAboutClick(Sender: TObject);
begin
  with TAboutForm.Create(self) do
    try
      ShowModal;
    finally
      Free;
    end;
end;

procedure TMainForm.TimerTimer(Sender: TObject);
var
  bd, bd2: PByte;
  Len, Len2: DWord;
  Status: Longint;
  Cnt: Integer;
begin
  for Cnt := 0 to Files.Count-1 do
    with TFileForm(Files[Cnt]) do
    begin
      Status := DSB.Status;

      if (Status and DSBSTATUS_BUFFERLOST <> 0) then
      //
      //  Restore the buffer, rewrite data, and play
      //
      try
        DSB.Restore;

        DSB.Lock( 0, cbSize, Pointer(bd), Len, Pointer(bd2), Len2, 0 );
        CopyMemory( bd, pbData, cbSize);
        DSB.Unlock( bd, Len, nil, 0 );

        if (fPlaying) then
          if (fLooped) then
            DSB.Play( DSBPLAY_LOOPING )
          else
            DSB.Play( 0 );

        Status := DSB.Status;
      except
        on EDirectSoundError do;
      end;

      UpdateStatus( Status );

      if (Status and DSBSTATUS_BUFFERLOST = 0) then
        if fPlaying and ((Status and DSBSTATUS_PLAYING) = 0) then
        begin
          StopDSound;
          buttonPlay.Caption := 'Play';
        end;

      if (Status and DSBSTATUS_BUFFERLOST <> 0) then fLost := True
      else fLost := False;
    end;
end;

procedure TMainForm.miFileExitClick(Sender: TObject);
begin
  Close;
end;


procedure TMainForm.miOptionsEnumerateDriversClick(Sender: TObject);
begin
  miOptionsEnumerateDrivers.Checked := not miOptionsEnumerateDrivers.Checked;
  if miOptionsEnumerateDrivers.Checked then
    MessageBox( Handle, 'Drivers will not be enumerated until DSSHOW is run again.',
      'DSShow', MB_OK );
end;


procedure TMainForm.miOptionsCheckLatencyClick(Sender: TObject);
var
  Cnt: Integer;
begin
	StopAllDSounds;

	// Now fake that we're on in each voice so the
	//timer will update the
	// strings in the window.
  for Cnt := 0 to Files.Count-1 do
    TFileForm( Files[Cnt] ).fPlaying := True;

  with TLatencyForm.Create(Self) do
    try
      ShowModal;
    finally
      Free;
    end;
end;


procedure TMainForm.miOptionsOutputTypeClick(Sender: TObject);
begin
  with TOutputForm.Create(self) do
    try
      ShowModal;
    finally
      Free;
    end;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;


procedure TMainForm.miFileOpenClick(Sender: TObject);
var
  FileName: String;
  nFileName: Integer;
  fSticky: Boolean;
  ff: TFileForm;
begin
  // Open the file, and check its format, etc.
  SetLength( FileName, MAX_PATH );
  if (OpenFileDialog( FileName, nFileName, fSticky)) then
  begin
    ff := TFileForm.Create(self);
    ff.FileName := Copy(FileName, 1, Pos(#0, FileName)-1);
    ff.Parent := self;
    ff.Align := alLeft;
    ff.fSticky := fSticky;
    ff.nFileName := nFileName;

    ff.Init;
  end;
end;

end.
