unit frmOutput;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, MMSystem;

type
  TOutputForm = class(TForm)
    Label1: TLabel;
    listboxType: TListBox;
    buttonOK: TButton;
    buttonCancel: TButton;
    buttonApply: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure buttonOKClick(Sender: TObject);
    procedure buttonCancelClick(Sender: TObject);
    procedure buttonApplyClick(Sender: TObject);
  private
    procedure IndexToFormat;
  public
  end;

var
  OutputForm: TOutputForm;

implementation

{$R *.DFM}

uses frmMain;


procedure TOutputForm.FormCreate(Sender: TObject);
var
  wfx: TWaveFormatEx;
  WaveStyle: Integer;
begin
  // Get the format.
  wfx := TMainForm(Owner).DS.Format;

  if (wfx.wFormatTag <> WAVE_FORMAT_PCM) then exit;

  // Check the channels
  case wfx.nChannels of
    1: WaveStyle := 0;
    2: WaveStyle := 1;
    else exit;
  end;

  // Check the bits...
  case wfx.wBitsPerSample of
    8: ;
    16: WaveStyle := WaveStyle or 2;
    else exit;
  end;

  // Check the rate.
  case wfx.nSamplesPerSec of
    8000: ;
  	11025: WaveStyle := WaveStyle or 4;
  	22050: WaveStyle := WaveStyle or 8;
  	44100: WaveStyle := WaveStyle or 12;
    else exit;
  end;

  listboxType.ItemIndex := WaveStyle;
end;

procedure TOutputForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;


// =======================================================================
//  This will convert an index (from a list box for instance)
//  to a format by passing
//  in the format to direct sound.
//
// =======================================================================
procedure TOutputForm.IndexToFormat;
var
  wfx: TWaveFormatEx;
begin
  wfx.wFormatTag := WAVE_FORMAT_PCM;
  if (listboxType.ItemIndex mod 2 = 0) then wfx.nChannels := 1
  else wfx.nChannels := 2;

  if (listboxType.ItemIndex mod 4 < 2) then
  begin
    wfx.nBlockAlign := 1*wfx.nChannels;
    wfx.wBitsPerSample := 8;
  end
  else
  begin
    wfx.nBlockAlign := 2*wfx.nChannels;
    wfx.wBitsPerSample := 16;
  end;

  wfx.nSamplesPerSec := 44100;
  if (listboxType.ItemIndex < 4) then wfx.nSamplesPerSec := 8000
  else if (listboxType.ItemIndex < 8) then wfx.nSamplesPerSec := 11025
    else if (listboxType.ItemIndex < 12) then wfx.nSamplesPerSec := 22050;

  wfx.nAvgBytesPerSec := wfx.nSamplesPerSec * wfx.nBlockAlign;
  wfx.cbSize := 0;

  TMainForm(Owner).DS.Format := wfx;
end;

procedure TOutputForm.buttonOKClick(Sender: TObject);
begin
  IndexToFormat;
  Close;
end;

procedure TOutputForm.buttonCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TOutputForm.buttonApplyClick(Sender: TObject);
begin
  IndexToFormat;
end;

end.
