unit frmMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Menus, Wave, DDirectSound, DSound, MMSystem,
  Registry, frmDriver, frmAbout;

const
  AppName = 'DirectSound Stream Sample Demo';

  MAX_PATH = 255;
  NUM_PLAY_NOTIFICATIONS = 4;	// number of notifications *not* including stop.

  PAN_MIN                 = 0;
  PAN_MAX                 = 800;
  PAN_CENTER              = 400;
  PAN_SHIFT               = -400;
  PAN_PAGESIZE            = 10;
  PAN_DIV                 = 10;
  PAN_MULT                = 10;
  VOL_SHIFT               = -400;
  VOL_MIN			            = 0;
  VOL_MAX                 = 400;
  VOL_PAGESIZE            = 10;
  VOL_DIV                 = 10;
  VOL_MULT                = 10;
  FREQ_MIN		            = 441;
  FREQ_MAX		            = 4410;
  FREQ_PAGESIZE		        = 100;
  FREQ_MULTIPLIER		      = 10;
  PROG_MIN		            = 0;
  PROG_MAX		            = 10000;
  PROG_MULTIPLIER         = 100;

  WM_DSSTREAM_DONE	      = WM_USER + $100;	// Make our own app messages
  WM_DSSTREAM_DEBUG	      = WM_USER + $101;
  WM_DSSTREAM_PROGRESS	  = WM_USER + $102;

  DSSTREAM_STOPF_NOREOPEN	= $0001;
  DSSTREAM_STOPF_NOEXIT	  = $0002;	// Prevent the stop code from calling exit


// Registry Key and Value names that allow us to retrive a path something like
// "C:\DXSDK\SDK\MEDIA", but matching the current install.
  RegKeyDirect3D = 'Software\Microsoft\Direct3D';
  RegValueD3DPath = 'D3D Path';
  RegKeyDSStream = 'Software\CBarbosa\DSStram';
  RegValueEnumDrivers = 'EnumDrivers';



type
  TWAVEINFOCA = record
    wfx: TWAVEFORMATEX;		    // Wave Format data structure */
    hmmio: HMMIO;		    // MM I/O handle for the WAVE */
    dwBufferSize: DWord;	    // Size of the entire buffer */
	  dwNotifySize: DWord;		// size of each notification period.
    dwNextWriteOffset: DWord;   // Offset to next buffer segment */
    dwProgress: DWord;	    // Used with above to show prog. */
	  dwLastPos: DWord;			// the last play position returned by GetCurrentPos().
    bDonePlaying: Boolean;	    // Signals early abort to timer */
    bLoopFile: Boolean;	    // Should we loop playback? */
    bFoundEnd: Boolean;	    // Timer found file end */
  end;

  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    miFile: TMenuItem;
    miFileOpen: TMenuItem;
    miFileClose: TMenuItem;
    miFileExit: TMenuItem;
    miPlay: TMenuItem;
    miStop: TMenuItem;
    miOptions: TMenuItem;
    miOptionsEnumerateDrivers: TMenuItem;
    miHelp: TMenuItem;
    miHelpAbout: TMenuItem;
    trackbarPan: TTrackBar;
    Bevel1: TBevel;
    trackbarVol: TTrackBar;
    trackbarFreq: TTrackBar;
    trackbarProgress: TTrackBar;
    labelPan: TLabel;
    labelVol: TLabel;
    labelFrequency: TLabel;
    labelProgress: TLabel;
    checkboxLooped: TCheckBox;
    buttonPlay: TButton;
    buttonStop: TButton;
    procedure FormCreate(Sender: TObject);
    procedure miHelpAboutClick(Sender: TObject);
    procedure buttonStopClick(Sender: TObject);
    procedure miFileOpenClick(Sender: TObject);
    procedure miFileCloseClick(Sender: TObject);
    procedure checkboxLoopedClick(Sender: TObject);
    procedure miOptionsEnumerateDriversClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure buttonPlayClick(Sender: TObject);
    procedure miFileExitClick(Sender: TObject);
    procedure trackbarPanChange(Sender: TObject);
    procedure miPlayClick(Sender: TObject);
    procedure miStopClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    Registry: TRegistry;
    CDStartPath: String;	        // The path to start the Open dialog in
    WaveFS: TFileStream;
    WaveSize, WaveOffset: Longint;
    wiWave: TWAVEINFOCA;
    bFileOpen, bPlaying, gfCloseOnDone: Boolean;
    uTimerID, uLastPercent: Longint;
    gdwFocus: Longint;
    FileTitle: String;
    FileBuffer: String;

    procedure GetMediaStartPath;
    procedure UpdateProgress( Progress: DWord );
    procedure OnPosition( Sender: TObject; Index: Longint );
    procedure SetupNotifications;
    procedure BuildTitleBarText;
    procedure UpdateFromControls;
    procedure ResetWavePlayer;
    procedure FillDataBuffer;
    procedure StreamBufferSetup;
    procedure DoStop( Param: Longint );
  public
    DS: TDirectSound;
    DSBStreamBuffer: TDirectSoundBuffer;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}


// ==========================================================
// SetupNotifications
//	Sets notifications and also creates the events.
// ==========================================================
procedure TMainForm.SetupNotifications;
var
  p: array [0..NUM_PLAY_NOTIFICATIONS] of DWord;
  Cnt: DWord;
begin
  for Cnt := 0 to NUM_PLAY_NOTIFICATIONS-1 do
    p[Cnt] := wiWave.dwNotifySize * (Cnt+1);
  Dec( p[NUM_PLAY_NOTIFICATIONS-1] );
  p[NUM_PLAY_NOTIFICATIONS] := DSBPN_OFFSETSTOP;

  DSBStreamBuffer.OnPosition := OnPosition;
  DSBStreamBuffer.SetNotificationPositions( NUM_PLAY_NOTIFICATIONS+1, @p );
end;


//****************************************************************************/
//* BuildTitleBar()                                                          */
//*                                                                          */
//*   Small routine to build and set the title bar text depending on whether */
//* or not a file is open.                                                   */
//****************************************************************************/
procedure TMainForm.BuildTitleBarText;
begin
  caption := 'DirectSound Stream Sample Application - ' + FileTitle;
end;



//********************************************************************************/
//* UpdateFromControls()                                                         */
//*                                                                              */
//*    This function gets all the required values from the DirectSoundBuffer and */
//* updates the screen interface controls.                                       */
//*                                                                              */
//********************************************************************************/
procedure TMainForm.UpdateFromControls;
begin
  // Set the volume and then the pan */
  if (DSBStreamBuffer <> nil) then
  begin
    //* Set the volume */
    labelVol.Caption := 'Volume: '
      + IntToStr((trackbarVol.Position + VOL_SHIFT) div VOL_DIV) + 'dB';
    DSBStreamBuffer.Volume := (trackbarVol.Position + VOL_SHIFT) * VOL_MULT;

    labelPan.Caption := 'Pan: '
      + IntToStr((trackbarPan.Position + PAN_SHIFT) div PAN_DIV) + 'dB';
    DSBStreamBuffer.Pan := (trackbarPan.Position + PAN_SHIFT) * PAN_MULT;

    labelFrequency.Caption := 'Frequency: '
      + IntToStr(trackbarFreq.Position * FREQ_MULTIPLIER) + 'Hz';
    DSBStreamBuffer.Frequency := trackbarFreq.Position * FREQ_MULTIPLIER;
  end;
end;


//*****************************************************************************/
//* ResetWavePlayer()                                                         */
//*                                                                           */
//*  Performs a subset of the above operations (in StreamBufferSetup). Things */
//* not done include creating a DSB and opening the file (it's already open). */
//*                                                                           */
//*****************************************************************************/
procedure TMainForm.ResetWavePlayer;
begin
  //* Seek to the data chunk */
  WaveFS.Position := WaveOffset;

  wiWave.bFoundEnd := FALSE;
	wiWave.dwNextWriteOffset := 0;

  FillDataBuffer;

  wiWave.bDonePlaying := FALSE;
  UpdateProgress( 0 );
end;


// =======================================================================
// GetMediaStartPath()                                                      */
//                                                                          */
//   This helper function attempts to get the media directory for Direct3D, */
// which is where all the installed DX wave files go. If it can't find that */
// it settles for the media sub-directory of the Windows directory.         */
// =======================================================================
procedure TMainForm.GetMediaStartPath;
begin
  Registry.RootKey := HKEY_LOCAL_MACHINE;
  if Registry.OpenKey( RegKeyDirect3D, False ) then
    if Registry.ValueExists( RegValueD3DPath ) then
      CDStartPath := Registry.ReadString( RegValueD3DPath )
    else
    begin
      SetLength( CDStartPath, MAX_PATH );
      GetWindowsDirectory( PChar(CDStartPath), MAX_PATH );
      CDStartPath := Copy(CDStartPath, 1, Pos(#0, CDStartPath)-1);
      if (CDStartPath[ Length(CDStartPath) ] <> '\') then
        CDStartPath := CDStartPath + '\';
      CDStartPath := CDStartPath + 'MEDIA';
    end;
  Registry.CloseKey;

  Registry.RootKey := HKEY_CURRENT_USER;
end;



procedure TMainForm.UpdateProgress( Progress: DWord );
var
  Percent: DWord;
begin
  if (WaveFS = nil) then
    Percent := 0
  else
  begin
    Progress := Progress mod DWord(WaveSize);
    Percent := (Progress * 100) div DWord(WaveSize);
  end;

  trackbarProgress.Position := Integer( Percent*PROG_MULTIPLIER );
  labelProgress.Caption := 'Progress: ' + IntToStr( Percent ) + '%';
end;


//*****************************************************************************/
//* StreamBufferSetup()                                                       */
//*                                                                           */
//* This function uses the filename stored in the global character array to*/
//* open a WAVE file. Then it creates a secondary DirectSoundBuffer object    */
//* which will later be used to stream that file from disk during playback.   */
//*                                                                           */
//*****************************************************************************/
procedure TMainForm.StreamBufferSetup;
var
  dsbd: TDSBufferDesc;
  nRem: DWord;
begin
  WaveFS := TFileStream.Create( FileBuffer, fmOpenRead );
  GetStreamWaveInfo( WaveFS, WaveSize, WaveOffset, wiWave.wfx );

  if( wiWave.wfx.wFormatTag <> WAVE_FORMAT_PCM ) then
  begin
    WaveFS.Free;
    WaveFS := nil;
    raise Exception.Create('The selected WAVE file is not a PCM encoded file and cannot be opened by this application.');
  end;

  // Calculate a buffer length 3 sec. long. This should be an integral number of the
	// number of bytes in one notification period.
	wiWave.dwNotifySize := wiWave.wfx.nSamplesPerSec * 3
    * wiWave.wfx.nBlockAlign;
	wiWave.dwNotifySize := wiWave.dwNotifySize div NUM_PLAY_NOTIFICATIONS;

	// the notify size should be an intergral multiple of the nBlockAlignvalue.
  nRem := wiWave.dwNotifySize mod wiWave.wfx.nBlockAlign;
	if (nRem <> 0) then
		wiWave.dwNotifySize := wiWave.dwNotifySize
      + (wiWave.wfx.nBlockAlign - nRem);

	wiWave.dwBufferSize := wiWave.dwNotifySize * NUM_PLAY_NOTIFICATIONS;

   //Create the secondary DirectSoundBuffer object to receive our sound data.
  dsbd.dwSize := sizeof( TDSBufferDesc );
  // Use new GetCurrentPosition() accuracy (DirectX 2 feature)
  dsbd.dwFlags := DSBCAPS_CTRLPOSITIONNOTIFY or DSBCAPS_CTRLDEFAULT
    or DSBCAPS_GETCURRENTPOSITION2 or gdwFocus;
  dsbd.dwBufferBytes := wiWave.dwBufferSize;
  dsbd.dwReserved := 0;

  //Set Format properties according to the WAVE file we just opened
  dsbd.lpwfxFormat := @(wiWave.wfx);

  DSBStreamBuffer := TDirectSoundBuffer.Create( DS, dsbd );

  wiWave.bFoundEnd := FALSE;
  wiWave.dwNextWriteOffset := 0;
  wiWave.bLoopFile := checkboxLooped.Checked;

  // Fill data in the buffer.
  FillDataBuffer;

  // we're set to play now.
  wiWave.bDonePlaying := FALSE;

  trackbarVol.Position := VOL_MAX;
  trackbarPan.Position := PAN_CENTER;
  trackbarFreq.Position := wiWave.wfx.nSamplesPerSec div FREQ_MULTIPLIER;

  UpdateProgress( 0 );
  UpdateFromControls;
end;


//*****************************************************************************/
//* FillDataBuffer()                                                          */
//*                                                                           */
//*   This function fills the sound buffer with a block of data, starting at  */
//* the current read position.  The entire buffer is filled.                  */
//*                                                                           */
//*****************************************************************************/
procedure TMainForm.FillDataBuffer;
var
  lpWrite1, lpWrite2: PChar;
  dwLen1, dwLen2: DWord;
  ActualBytesWritten, BytesLeft: DWord;
  dwBytes: DWord;
  uWritten: DWord;
begin
	dwBytes := wiWave.dwBufferSize;

	// This is the initial read. So we fill the entire buffer.
	// This will not wrap around so the 2nd pointers will be NULL.
  DSBStreamBuffer.Lock( 0, dwBytes, Pointer(lpWrite1), dwLen1,
    Pointer(lpWrite2), dwLen2, 0 );
  try
    BytesLeft := WaveSize - (WaveFS.Position - WaveOffset);
    if (BytesLeft > dwLen1) then
      BytesLeft := dwLen1;
    // if the number of bytes written is less than the
    // amount we requested, we have a short file.
    ActualBytesWritten := WaveFS.Read( lpWrite1^, BytesLeft );
    if (ActualBytesWritten < dwLen1) then
    begin
      if not wiWave.bLoopFile then
      begin
        // we set the bFoundEnd flag if the length is less than
        // one notify period long which is when the first notification comes in.
        // The next notification will then call send a message to process a stop.
        if (ActualBytesWritten < DWord(wiWave.dwNotifySize)) then
          wiWave.bFoundEnd := TRUE;

        // Fill in silence for the rest of the buffer.
        if (wiWave.wfx.wBitsPerSample = 8) then
          FillMemory( lpWrite1 + ActualBytesWritten,
            dwLen1 - ActualBytesWritten, 128)
        else
          FillMemory( lpWrite1 + ActualBytesWritten,
            dwLen1 - ActualBytesWritten, 0);
      end
      else
      begin
        uWritten := ActualBytesWritten;	// from previous call above.

        // we are looping.
        while (uWritten < dwLen1) do
        begin	// this will keep reading in until the buffer is full. For very short files.
          WaveFS.Position := WaveOffset;
          WaveFS.Read( (lpWrite1 + uWritten)^, dwLen1-uWritten );
          uWritten := uWritten + ActualBytesWritten;
        end;
      end;
    end;
  finally
  	// now unlock the buffer.
	  DSBStreamBuffer.Unlock( lpWrite1, dwLen1,	nil, 0 );
  end;

	wiWave.dwNextWriteOffset := wiWave.dwNextWriteOffset + dwLen1;
	// this is a circular buffer. Do mod buffersize.
	if (wiWave.dwNextWriteOffset >= wiWave.dwBufferSize) then
		wiWave.dwNextWriteOffset := wiWave.dwNextWriteOffset - wiWave.dwBufferSize;

	wiWave.dwProgress := 0;
	wiWave.dwLastPos := 0;
end;


procedure TMainForm.DoStop( Param: Longint );
begin
  if (gfCloseOnDone) then
    wiWave.bDonePlaying := True;

  if ( bPlaying ) then
  begin
    bPlaying := FALSE;
    DSBStreamBuffer.Stop;
    miStop.Enabled := False;
    buttonStop.Enabled := False;
  end;

  UpdateProgress( 0 );

  // Short circuit to allow command-line forced shutdown
  if (( Param and DSSTREAM_STOPF_NOREOPEN ) = 0) and (not gfCloseOnDone) then
  begin
    ResetWavePlayer;
    miPlay.Enabled := True;
    buttonPlay.Enabled := True;
  end
  else
  begin
    if( bFileOpen ) then
    begin
      DSBStreamBuffer.Free;
      DSBStreamBuffer := nil;

      WaveFS.Free;
      WaveFS := nil;

      bFileOpen := FALSE;

      // The file is closed, so disable the close option
      miFileClose.Enabled := False;
      miPlay.Enabled := False;
      buttonPlay.Enabled := False;
      if gfCloseOnDone and	(( Param and DSSTREAM_STOPF_NOEXIT ) = 0) then
        miFileClose.OnClick( self );
    end;
  end;
end;


procedure TMainForm.OnPosition( Sender: TObject; Index: Longint );
var
  lpWrite1, lpWrite2: PChar;
  dwWrite1, dwWrite2, dwPlayPos, dwPlayed: DWord;
  cbWritten, cbActual, BytesLeft: DWord;
begin
  if (wiWave.bDonePlaying = TRUE) then
  begin
			UpdateProgress( 0 );
      DoStop( 0 );
      exit;
  end;

  dwPlayPos := DSBStreamBuffer.PlayPosition;
	if (dwPlayPos < wiWave.dwLastPos) then
		dwPlayed := wiWave.dwBufferSize - wiWave.dwLastPos + dwPlayPos
	else
		dwPlayed := dwPlayPos - wiWave.dwLastPos;
	Inc( wiWave.dwProgress, dwPlayed );
	wiWave.dwLastPos := dwPlayPos;
  UpdateProgress( wiWave.dwProgress );

  if (Index <> NUM_PLAY_NOTIFICATIONS) then
  begin
    // If the entire file has been read into the buffer, bFoundEnd will be TRUE.
    if (not wiWave.bFoundEnd) then
    begin
			// Lock the buffer.
			DSBStreamBuffer.Lock(	wiWave.dwNextWriteOffset, wiWave.dwNotifySize,
        Pointer(lpWrite1), dwWrite1,	Pointer(lpWrite2), dwWrite2, 0);

      try
        BytesLeft := WaveSize - (WaveFS.Position - WaveOffset);
        if (BytesLeft > dwWrite1) then
          BytesLeft := dwWrite1;

        // if the number of bytes written is less than the
        // amount we requested, we have a short file.
        cbActual := WaveFS.Read( lpWrite1^, BytesLeft );
        if (cbActual < dwWrite1) then
        begin
          if not wiWave.bLoopFile then
          begin
            wiWave.bFoundEnd := TRUE;

            // Fill in silence for the rest of the buffer.
            if (wiWave.wfx.wBitsPerSample = 8) then
              FillMemory( lpWrite1 + cbActual,
                dwWrite1 - cbActual, 128)
            else
              FillMemory( lpWrite1 + cbActual,
                dwWrite1 - cbActual, 0);
          end
          else
          begin
            cbWritten := cbActual;	// from previous call above.

            // we are looping.
            while (cbWritten < dwWrite1) do
            begin	// this will keep reading in until the buffer is full. For very short files.
              WaveFS.Position := WaveOffset;
              cbActual := WaveFS.Read( (lpWrite1 + cbWritten)^, dwWrite1-cbWritten );
              cbWritten := cbWritten + cbActual;
            end;
          end;
        end;
      finally
        // now unlock the buffer.
        DSBStreamBuffer.Unlock( lpWrite1, dwWrite1,	nil, 0 );
      end;

      Inc( wiWave.dwNextWriteOffset, dwWrite1 );
      // this is a circular buffer. Do mod buffersize.
      if (wiWave.dwNextWriteOffset >= wiWave.dwBufferSize) then
        Dec( wiWave.dwNextWriteOffset, wiWave.dwBufferSize );
    end
    else
    begin
      // Allow the rest of the bytes to be played and fill here
      // with silence. The next notification will quit the while loop.
      DSBStreamBuffer.Lock( wiWave.dwNextWriteOffset,
        wiWave.dwNotifySize, Pointer(lpWrite1), dwWrite1,
          Pointer(lpWrite2), dwWrite2, 0);
      try
        // Fill in silence for the rest of the buffer.
        if (wiWave.wfx.wBitsPerSample = 8) then
          FillMemory( lpWrite1, dwWrite1, 128)
        else
          FillMemory( lpWrite1, dwWrite1, 0);
      finally
        DSBStreamBuffer.Unlock( lpWrite1, dwWrite1, nil, 0 );
      end;

      // We don't want to cut off the sound before it's done playing.
      // If doneplaying is set, the next notification event will post a stop message.
      if (DWord(WaveSize) > wiWave.dwNotifySize) and
        (wiWave.dwProgress >= DWord(WaveSize) - wiWave.dwNotifySize) then
        wiWave.bDonePlaying := TRUE
      else	// for short files.
        if (wiWave.dwProgress >= DWord(WaveSize)) then
          wiWave.bDonePlaying := TRUE;
    end;
  end
  else
  begin
    // Set bDonePlaying to True.
    wiWave.bDonePlaying := TRUE;
    // call stop.
    DoStop( 0 );
  end;
end;


procedure TMainForm.FormCreate(Sender: TObject);
var
  G: PGUID;
begin
  Registry := TRegistry.Create;

  // Set up the global File...Open dialog's start directory
  GetMediaStartPath;

  // Now set up all the direct sound stuff...
  G := nil;
  Registry.OpenKey( RegKeyDSStream, True );
  if Registry.ValueExists( RegValueEnumDrivers ) then
    miOptionsEnumerateDrivers.Checked := Registry.ReadBool( RegValueEnumDrivers );
  Registry.CloseKey;

  if miOptionsEnumerateDrivers.Checked then
    with TDriverForm.Create( self ) do
      try
        ShowModal;
        if GUIDPtr <> nil then
        begin
          New( G );
          G^ := GUIDPtr^;
        end;
      finally
        Free;
      end;

  DS := TDirectSound.Create( G, 0 );

  if (G <> nil) then
    Dispose( G );

  // Note we need to set the level to be priority to set the
  // format of the primary buffer
  DS.SetCooperativeLevel( Handle, DSSCL_NORMAL );

  bFileOpen := False;
  bPlaying := False;
  gfCloseOnDone := False;
  uTimerID := 0;
  uLastPercent := 100;
  gdwFocus := 0;

//	LoadFromCommandLine( lpCmdLine );
end;

procedure TMainForm.miHelpAboutClick(Sender: TObject);
begin
  with TAboutForm.Create( self ) do
    try
      ShowModal;
    finally
      Free;
    end;
end;

procedure TMainForm.buttonStopClick(Sender: TObject);
begin
  DoStop( 0 );
end;


procedure TMainForm.miFileOpenClick(Sender: TObject);
begin
  with TOpenDialog.Create(self) do
    try
      Filter := 'Wave Files (*.wav)|*.WAV';
      Options := [ofFileMustExist, ofHideReadOnly];
      InitialDir := CDStartPath;
      Title := 'DirectSound Stream';
      if not Execute then exit;

			// Copy the directory name we opened from so that we start there
			// next time we open the dialog box...
      CDStartPath := ExtractFilePath( FileName );

      FileTitle := ExtractFileName( FileName );
      FileBuffer := FileName;

      if ( bFileOpen ) then
      begin
  			// Before we force a close, disable auto close
				// because the user has picked a new file and is
				// obviously not ready for us to go away yet.
        gfCloseOnDone := FALSE;

				//* Need to close the previous file before we open a new one.  The best
        //* way to do this is by faking a menu command, so that we only have the
				//* actual code in one place and it can easily be changed.
        miFileClose.OnClick( self );
      end;

			StreamBufferSetup;

      bFileOpen := TRUE;
      miPlay.Enabled := True;
      buttonPlay.Enabled := True;
      miFileClose.Enabled := True;

      BuildTitleBarText;
    finally
      Free;
    end;
end;

procedure TMainForm.miFileCloseClick(Sender: TObject);
begin
  DoStop( DSSTREAM_STOPF_NOREOPEN );
  BuildTitleBarText;
end;

procedure TMainForm.checkboxLoopedClick(Sender: TObject);
begin
  wiWave.bLoopFile := checkboxLooped.Checked;
  if not bPlaying and bFileOpen then
    ResetWavePlayer;
end;

procedure TMainForm.miOptionsEnumerateDriversClick(Sender: TObject);
begin
  miOptionsEnumerateDrivers.Checked := not miOptionsEnumerateDrivers.Checked;
  if miOptionsEnumerateDrivers.Checked then
    MessageBox( Handle, 'Drivers will not be enumerated until DSSHOW is run again.',
      'DSShow', MB_OK );
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  DoStop( DSSTREAM_STOPF_NOREOPEN or DSSTREAM_STOPF_NOEXIT );

  // Destroy the direct sound
  DS.Free;

  Registry.OpenKey( RegKeyDSStream, True );
  Registry.WriteBool( RegValueEnumDrivers, miOptionsEnumerateDrivers.Checked );
  Registry.CloseKey;
  Registry.Free;
end;

procedure TMainForm.buttonPlayClick(Sender: TObject);
begin
  if (bPlaying) then
  begin
    gfCloseOnDone := False;
    DoStop( 0 );
  end;

  if bFileOpen and (DSBStreamBuffer <> nil) then
  begin
    SetupNotifications();

    // Ensure that position is at 0, ready to go
    DSBStreamBuffer.PlayPosition := 0;

    wiWave.bDonePlaying := FALSE;
    DSBStreamBuffer.Play( DSBPLAY_LOOPING );
  end
  else
  begin
    bPlaying := FALSE;
    exit;
  end;

  bPlaying := TRUE;
  buttonPlay.Enabled := False;
  miPlay.Enabled := False;
  miStop.Enabled := True;
  buttonStop.Enabled := True;
end;

procedure TMainForm.miFileExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.trackbarPanChange(Sender: TObject);
begin
  UpdateFromControls;
end;


procedure TMainForm.miPlayClick(Sender: TObject);
begin
  buttonPlay.OnClick(Sender);
end;

procedure TMainForm.miStopClick(Sender: TObject);
begin
  buttonStop.OnClick(Sender);
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
  if bFileOpen and (DSBStreamBuffer <> nil) then
  begin
    SetupNotifications();

    // Ensure that position is at 0, ready to go
    DSBStreamBuffer.PlayPosition := 0;

    wiWave.bDonePlaying := FALSE;
    DSBStreamBuffer.Play( DSBPLAY_LOOPING );
  end
  else
  begin
    bPlaying := FALSE;
    exit;
  end;

  bPlaying := TRUE;
  buttonPlay.Enabled := False;
  miPlay.Enabled := False;
  miStop.Enabled := True;
  buttonStop.Enabled := True;

  if (gfCloseOnDone) then
    wiWave.bDonePlaying := True;

  if ( bPlaying ) then
  begin
    bPlaying := FALSE;
    DSBStreamBuffer.Stop;
    miStop.Enabled := False;
    buttonStop.Enabled := False;
  end;

  UpdateProgress( 0 );

  // Short circuit to allow command-line forced shutdown
  ResetWavePlayer;
  miPlay.Enabled := True;
  buttonPlay.Enabled := True;
end;

initialization

end.
