unit frmMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, DDirectSound, DSound, Registry, MMSystem,
  CommDlg, Wave, StdCtrls;

const
  NUMSOUNDS = 4;

type
  TMainForm = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button1Click(Sender: TObject);
  private
    function NewDirectSoundBuffer( FileName: String;
      var pwfx: PWAVEFORMATEX ): TDirectSoundBuffer;
  public
    DS: TDirectSound;
    DSB: array [0..NUMSOUNDS-1] of  TDirectSoundBuffer;
    pwfx: array [0..NUMSOUNDS-1] of PWAVEFORMATEX;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}


procedure TMainForm.FormCreate(Sender: TObject);
var
  Cnt: Integer;
  s: TFileStream;
  Size, Offset: Longint;
  wi: TWaveFormatEx;
  p: Pointer;
begin
  s := TFileStream.Create( 'sound1.wav', fmOpenRead );
  GetMem(p, 13000);
  s.Read( p^, s.Size );
  GetMemoryWaveInfo( p, s.Size, Size, Pointer(Offset), wi );
  FreeMem( p );
  s.Free;

  DS := TDirectSound.Create( nil, 0 );

  DS.SetCooperativeLevel( Handle, DSSCL_NORMAL);

  for Cnt := 0 to NUMSOUNDS-1 do
    DSB[Cnt] := NewDirectSoundBuffer( 'Sound' + Char(Integer('1')+ Cnt) + '.wav',
      pwfx[Cnt] );
end;


function TMainForm.NewDirectSoundBuffer( FileName: String;
  var pwfx: PWAVEFORMATEX ): TDirectSoundBuffer;
var
  bd, bd2: Pointer;
  dsbd: TDSBufferDesc;
  Len, Len2: DWord;
  pbData: Pointer;
  cbSize: Longint;
begin
  WaveLoadFile( Filename, cbSize, pwfx, pbData );

  try
    try
      // Set up the direct sound buffer.
      ZeroMemory( @dsbd, sizeof(dsbd) );
      dsbd.dwSize := sizeof(TDSBUFFERDESC);
      dsbd.dwFlags := DSBCAPS_STATIC;
      dsbd.dwBufferBytes := cbSize;
      dsbd.lpwfxFormat := pwfx;

      Result := TDirectSoundBuffer.Create( DS, dsbd );

      Result.Lock( 0, cbSize, bd, Len, bd2, Len2, 0 );
      try
        CopyMemory( bd, pbData, cbSize );
      finally
        // Ok, now unlock the buffer, we don't need it anymore.
        Result.Unlock( bd, cbSize, nil, 0 );
      end;
    except
      on EDirectSoundError do
      begin
        FreeMem( pwfx );
        Result.Free;
        raise;
      end;
    end;
  finally
    FreeMem( pbData );
  end;
end;


procedure TMainForm.FormDestroy(Sender: TObject);
var
  Cnt: Integer;
begin
  for Cnt := 0 to NUMSOUNDS-1 do
    FreeMem( pwfx[Cnt] );

  DS.Free;
end;


procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;


procedure TMainForm.Button1Click(Sender: TObject);
begin
  DSB[TButton(Sender).Tag].PlayPosition := 0;
  DSB[TButton(Sender).Tag].Play( 0 );
end;

end.
